;---------------------------------------------------------------------------
;
;		   Standard Operating System Definitions
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;---------------------------------------------------------------------------

COPYRIGHT		EQU	<"Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.">
BUILD			EQU	<"Built ", ??date, " ", ??time>
CRLF			EQU	<13, 10>
BELL			EQU	<7>
NULL			EQU	<0>

;---------------------------------------------------------------------------
; Intel 386+ CPU Equates and Structures
;---------------------------------------------------------------------------

EFLAGS_CF		EQU	00000001h
EFLAGS_PF		EQU	00000004h
EFLAGS_AF		EQU	00000010h
EFLAGS_ZF		EQU	00000040h
EFLAGS_SF		EQU	00000080h
EFLAGS_TF		EQU	00000100h
EFLAGS_IF		EQU	00000200h
EFLAGS_DF		EQU	00000400h
EFLAGS_OF		EQU	00000800h
EFLAGS_IOPL		EQU	00003000h
EFLAGS_IOPL0		EQU	00000000h
EFLAGS_IOPL1		EQU	00001000h
EFLAGS_IOPL2		EQU	00002000h
EFLAGS_IOPL3		EQU	00003000h
EFLAGS_NT		EQU	00004000h
EFLAGS_RF		EQU	00010000h
EFLAGS_VM		EQU	00020000h
EFLAGS_AC		EQU	00040000h
EFLAGS_VIF		EQU	00080000h
EFLAGS_VIP		EQU	00100000h
EFLAGS_ID		EQU	00200000h
EFLAGS_RESET		EQU	00000002h

IOPL0			EQU	00000000h
IOPL1			EQU	00001000h
IOPL2			EQU	00002000h
IOPL3			EQU	00003000h

CR0_PE			EQU	00000001h
CR0_MP			EQU	00000002h
CR0_EM			EQU	00000004h
CR0_TS			EQU	00000008h
CR0_ET			EQU	00000010h
CR0_WT			EQU	20000000h
CR0_CD			EQU	40000000h
CR0_PG			EQU	80000000h

CR4_VME			EQU	00000001h
CR4_PVI			EQU	00000002h
CR4_TSD			EQU	00000004h
CR4_DE			EQU	00000008h
CR4_PSE			EQU	00000010h
CR4_PAE			EQU	00000020h
CR4_MCE			EQU	00000040h
CR4_PGE			EQU	00000080h
CR4_PCE			EQU	00000100h

; Stack Frame for "PUSHAD"

PUSHAD_FRAME		STRUC
pushad_EDI		DD	?
pushad_ESI		DD	?
pushad_EBP		DD	?
pushad_ESP		DD	?
pushad_EBX		DD	?
pushad_EDX		DD	?
pushad_ECX		DD	?
pushad_EAX		DD	?
PUSHAD_FRAME		ENDS

; Stack Frame for "V86" Interrupts

V86_FRAME		STRUC
v86_EIP			DD	?
v86_CS			DD	?
v86_EFLAGS		DD	?
v86_ESP			DD	?
v86_SS			DD	?
v86_ES			DD	?
v86_DS			DD	?
v86_FS			DD	?
v86_GS			DD	?
V86_FRAME		ENDS

; Stack Frame for our Exception Handlers

EXP_FRAME		STRUC
exp_Num			DD	?
exp_Error		DD	?
EXP_FRAME		ENDS

; Stack Frame for our IRQ Handlers

IRQ_FRAME		STRUC
irq_Num			DD	?
IRQ_FRAME		ENDS

; KERNEL32 Arguments

KERNEL_ARGS		STRUC
kaLibraryBase		DD	?
kaLibrarySize		DD	?
kaReclaimBase		DD	?
kaReclaimSize		DD	?
kaMDTBase		DD	?
kaMDTSize		DD	?
kaPSP			DD	?
kaEnvironment		DD	?
KERNEL_ARGS		ENDS

; Global Descriptor Table

GDT			STRUC
gdtLimit		DW	?
gdtBase_00_15		DW	?
gdtBase_16_23		DB	?
gdtType			DB	?
gdtMisc			DB	?
gdtBase_24_31		DB	?
GDT			ENDS

GDTR			STRUC
gdtrLimit		DW	?
gdtrBase		DD	?
GDTR			ENDS

; Interrupt Descriptor Table

IDT			STRUC
idtOfsLow		DW	?
idtSelector		DW	?
idtNull			DB	?
idtType			DB	?
idtOfsHigh		DW	?
IDT			ENDS

IDTR			STRUC
idtrLimit		DW	?
idtrBase		DD	?
IDTR			ENDS

; 32-bit Task State Segment

TSS			STRUC
tssLink			DW	?, ?
tssESP0			DD	?
tssSS0			DD	?
tssESP1			DD	?
tssSS1			DD	?
tssESP2			DD	?
tssSS2			DD	?
tssCR3			DD	?
tssEIP			DD	?
tssEFLAGS		DD	?
tssEAX			DD	?
tssECX			DD	?
tssEDX			DD	?
tssEBX			DD	?
tssESP			DD	?
tssEBP			DD	?
tssESI			DD	?
tssEDI			DD	?
tssES			DD	?
tssCS			DD	?
tssSS			DD	?
tssDS			DD	?
tssFS			DD	?
tssGS			DD	?
tssLDT			DW	?, ?
tssTrap			DW	?
tssIOPB			DW	?
TSS			ENDS

PAGE_PRESENT		EQU	000000000001b
PAGE_WRITABLE		EQU	000000000010b
PAGE_USER		EQU	000000000100b
PAGE_WT			EQU	000000001000b
PAGE_CD			EQU	000000010000b
PAGE_ACCESSED		EQU	000000100000b
PAGE_DIRTY		EQU	000001000000b
PAGE_GLOBAL		EQU	000010000000b
PAGE_SIZE		EQU	000100000000b
PAGE_AVL1		EQU	001000000000b
PAGE_AVL2		EQU	010000000000b
PAGE_AVL3		EQU	100000000000b

PAGE_RING0		EQU	PAGE_PRESENT + PAGE_WRITABLE
PAGE_RING3		EQU	PAGE_PRESENT + PAGE_WRITABLE + PAGE_USER

DESC_ACCESSED		EQU	01h
DESC_PRESENT		EQU	80h

DESC_LDT		EQU	82h
DESC_TSS		EQU	89h
DESC_TSS_BUSY		EQU	8Bh

DESC_TASK_GATE		EQU	85h
DESC_CALL_GATE		EQU	8Ch
DESC_INTR_GATE		EQU	8Eh
DESC_TRAP_GATE		EQU	8Fh

DESC_DATA		EQU	92h
DESC_CODE		EQU	9Ah

DPL0			EQU	0 SHL 5
DPL1			EQU	1 SHL 5
DPL2			EQU	2 SHL 5
DPL3			EQU	3 SHL 5

RPL0			EQU	0
RPL1			EQU	1
RPL2			EQU	2
RPL3			EQU	3

; standard GDT entries

CODE16			EQU	0008h
DATA16			EQU	0010h
CODE32			EQU	0018h
DATA32			EQU	0020h
CODE32_RING3		EQU	0028h + RPL3
DATA32_RING3		EQU	0030h + RPL3
KERNEL_TSS		EQU	0038h

;---------------------------------------------------------------------------
; Thread Memory -- Fixed Locations
;---------------------------------------------------------------------------

THREAD_MEM		EQU	0FF800000h
THREAD_CPU		EQU	0FF800100h
THREAD_FPU		EQU	0FF800200h
THREAD_ESP_MIN		EQU	0FF800400h
THREAD_ESP_MAX		EQU	0FF801000h

THREAD_REG_EAX		EQU	0FF800100h
THREAD_REG_EBX		EQU	0FF800104h
THREAD_REG_ECX		EQU	0FF800108h
THREAD_REG_EDX		EQU	0FF80010Ch
THREAD_REG_ESI		EQU	0FF800110h
THREAD_REG_EDI		EQU	0FF800114h
THREAD_REG_EBP		EQU	0FF800118h
THREAD_REG_ESP		EQU	0FF80011Ch

THREAD_REG_DS		EQU	0FF800120h
THREAD_REG_ES		EQU	0FF800124h
THREAD_REG_FS		EQU	0FF800128h
THREAD_REG_GS		EQU	0FF80012Ch
THREAD_REG_CS		EQU	0FF800130h
THREAD_REG_SS		EQU	0FF800134h

THREAD_REG_EFLAGS	EQU	0FF800140h
THREAD_REG_EIP		EQU	0FF800144h
THREAD_REG_ESP0		EQU	0FF800148h

THREAD_REG_MM0		EQU	0FF800218h
THREAD_REG_MM1		EQU	0FF80021Ch
THREAD_REG_MM2		EQU	0FF800228h
THREAD_REG_MM3		EQU	0FF800230h
THREAD_REG_MM4		EQU	0FF800238h
THREAD_REG_MM5		EQU	0FF800240h
THREAD_REG_MM6		EQU	0FF800248h
THREAD_REG_MM7		EQU	0FF800250h

;---------------------------------------------------------------------------
; Memory Areas
;---------------------------------------------------------------------------

USER_MEM		EQU	000000000h
USER_MEM_END		EQU	07FFFFFFFh
USER_MEM_SIZE		EQU	080000000h

SHARED_MEM		EQU	080000000h
SHARED_MEM_END		EQU	0BFFFFFFFh
SHARED_MEM_SIZE		EQU	040000000h

KERNEL_MEM		EQU	0C0000000h
KERNEL_MEM_END		EQU	0FF7FFFFFh
KERNEL_MEM_SIZE		EQU	03F800000h

KERNEL_PAGE_MAP		EQU	0FFC00000h
KERNEL_PAGE_DIR		EQU	0FFFFF000h

; some useful opcodes

FAR_JUMP		EQU	0EAh
FAR_CALL		EQU	09Ah
FAR_JUMP_32		EQU	0EA66h
FAR_CALL_32		EQU	09A66h

NEAR_JUMP		EQU	0E9h
NEAR_CALL		EQU	0E8h

;---------------------------------------------------------------------------
; Loader Memory Detection storage structure
;---------------------------------------------------------------------------

MDT			STRUC
mdtBase			DD	?
mdtSize			DD	?
MDT			ENDS

MEM_SMAP_FREE		EQU	1

MEM_SMAP_ENTRY		STRUC
memSmapBaseLo		DD	?
memSmapBaseHi		DD	?
memSmapSizeLo		DD	?
memSmapSizeHi		DD	?
memSmapType		DD	?
MEM_SMAP_ENTRY		ENDS

;---------------------------------------------------------------------------
; List & Queue Structures
;---------------------------------------------------------------------------

LIST_HEAD	STRUC
listHead	DD	?
listTail	DD	?
LIST_HEAD	ENDS

LIST_ITEM	STRUC
listPrev	DD	?
listNext	DD	?
LIST_ITEM	ENDS

;---------------------------------------------------------------------------
; Memory Manager 'Physical Range' Item
;---------------------------------------------------------------------------

MM_PAGE_RANGE	STRUC
mmList		LIST_ITEM <>
mmBase		DD	?
mmSize		DD	?
MM_PAGE_RANGE	ENDS

;---------------------------------------------------------------------------
; Kernel API Structures, Equates and Macros
;---------------------------------------------------------------------------

API_DYNAMIC		EQU	80000000h	; Dyna Bit (Table/Macro)
API_JUMP		EQU	00008000h	; Jump Bit (Macro)
API_USER		EQU	00008000h	; User Bit (Table)

API_TABLE		STRUC
apiProcTable		DD	?
apiProcCount		DD	?
apiListItem		LIST_ITEM <>
API_TABLE		ENDS

API_ENTRY		STRUC
apiProcID		DD	?
apiProcAddr		DD	?
API_ENTRY		ENDS

;---------------------------------------------------------------------------
; APIcall and APIjmp Macros
; =========================
;
; Usage:
;
;	APIcall	MODULE_ServiceName
;
;		or
;
;	APIjmp 	MODULE_ServiceName
;
;---------------------------------------------------------------------------

APIcall	MACRO	???procID:REQ
	int	40h
	DD	???procID AND 7FFF7FFFh
	ENDM

APIjmp	MACRO	???procID:REQ
	int	40h
	DD	???procID AND 7FFF7FFFh OR API_JUMP
	ENDM

APIproc MACRO	???procID:REQ, ???label:REQ
???label:
	int	40h
	DD	???procID AND 7FFF7FFFh OR API_JUMP
	nop
	nop
	ENDM

;---------------------------------------------------------------------------
; Reference external using easier name -- "EXTERN"
;---------------------------------------------------------------------------

EXTERN MACRO ???symbol:REQ
	EXTRN	???symbol
	ENDM

;---------------------------------------------------------------------------
; Procedure Definition
;---------------------------------------------------------------------------

DefProc MACRO procname:REQ

procname PROC NEAR

debugISegment SEGMENT
	DD	OFFSET procname
	DD	OFFSET ???&procname
debugISegment ENDS

debugDSegment SEGMENT
???&procname LABEL
     	DB	"&procname", NULL
debugDSegment ENDS

	ENDM

EndProc MACRO procname:REQ

procname ENDP

	ENDM

;---------------------------------------------------------------------------
; Breakpoint
;---------------------------------------------------------------------------

BREAKPOINT MACRO
	int	3
	ENDM

;---------------------------------------------------------------------------
; Para/Page/Size Alignment
;---------------------------------------------------------------------------

_PARA			EQU	00000010h
_PAGE			EQU	00001000h
_PAGE_4KB		EQU	00001000h
_PAGE_2MB		EQU	00200000h
_PAGE_4MB		EQU	00400000h
_1MEG			EQU	00100000h
_2MEG			EQU	00200000h
_4MEG			EQU	00400000h

_16KB			EQU	00004000h
_32KB			EQU	00008000h
_64KB			EQU	00010000h

_PARA_ALIGN		EQU	_PARA
_PAGE_ALIGN		EQU	_PAGE
_1MEG_ALIGN		EQU	_1MEG
_4MEG_ALIGN		EQU	_4MEG

PARA_ALIGN MACRO ???dest:REQ
	add	???dest, 00000000Fh
	and	???dest, 0FFFFFFF0h
	ENDM

PAGE_ALIGN MACRO ???dest:REQ
	add	???dest, 000000FFFh
	and	???dest, 0FFFFF000h
	ENDM

SIZE_ALIGN MACRO ???target:REQ, ???source:REQ
	add	???target, ???source
	not	???source
	and	???target, ???source
	not	???source
	ENDM

TESTD	MACRO ???op1:REQ, ???op2:REQ
	testflag DWORD PTR ???op1, ???op2
	ENDM

ORD	MACRO ???op1:REQ, ???op2:REQ
	setflag DWORD PTR ???op1, ???op2
	ENDM

ANDD	MACRO ???op1:REQ, ???op2:REQ
	maskflag DWORD PTR ???op1, ???op2
	ENDM

XORD	MACRO ???op1:REQ, ???op2:REQ
	flipflag DWORD PTR ???op1, ???op2
	ENDM

;---------------------------------------------------------------------------
; Load flat data pointer from "dataSegment"
;---------------------------------------------------------------------------

LOAD32	MACRO	???reg:REQ, ???item:REQ, ???seg

IFB	<???seg>
	mov	???reg, dataSegment
ELSE
	mov	???reg, ???seg
ENDIF
	shl	???reg, 4
	add	???reg, LARGE OFFSET ???item

	ENDM

;---------------------------------------------------------------------------
; 16-bit FAR Call and Jump
;---------------------------------------------------------------------------

CALLF	MACRO	???seg, ???ofs
	DB	09Ah
	DW	???ofs, ???seg
	ENDM

JMPF	MACRO	???seg, ???ofs
	DB	0EAh
	DW	???ofs, ???seg
	ENDM

;---------------------------------------------------------------------------
; 16-bit & 32-bit FAR Transition Jumps
;---------------------------------------------------------------------------

JMP16	MACRO	???label
	DB	066h
	DB	0EAh
	DW	OFFSET ???label
	DW	CODE16
	ENDM	

JMP32	MACRO	???label
	DB	066h
	DB	0EAh
	DD	OFFSET ???label
	DW	CODE32
	ENDM	

;---------------------------------------------------------------------------
; 8259 PIC symbols for readability
;---------------------------------------------------------------------------

IBM_PIC_BASE_1		EQU	008h
IBM_PIC_BASE_2		EQU	070h
IBM_PIC_BASE		EQU	07008h

STD_PIC_BASE_1		EQU	020h
STD_PIC_BASE_2		EQU	028h
STD_PIC_BASE		EQU	02820h

PIC_PORT20		EQU	020h
PIC_PORT21		EQU	021h

PIC_PORTA0		EQU	0A0h
PIC_PORTA1		EQU	0A1h

PIC_ICW1		EQU	11h
PIC_ICW2		EQU	?
PIC_ICW3_A		EQU	04h
PIC_ICW3_B		EQU	02h
PIC_ICW4		EQU	01h

PIC_IMR_NONE		EQU	00000000b
PIC_IMR_ALL		EQU	11111111b

PIC_IMR_IRQ0		EQU	00000001b
PIC_IMR_IRQ1		EQU	00000010b
PIC_IMR_IRQ2		EQU	00000100b
PIC_IMR_IRQ3		EQU	00001000b
PIC_IMR_IRQ4		EQU	00010000b
PIC_IMR_IRQ5		EQU	00100000b
PIC_IMR_IRQ6		EQU	01000000b
PIC_IMR_IRQ7		EQU	10000000b
PIC_IMR_IRQ8		EQU	00000001b
PIC_IMR_IRQ9		EQU	00000010b
PIC_IMR_IRQ10		EQU	00000100b
PIC_IMR_IRQ11		EQU	00001000b
PIC_IMR_IRQ12		EQU	00010000b
PIC_IMR_IRQ13		EQU	00100000b
PIC_IMR_IRQ14		EQU	01000000b
PIC_IMR_IRQ15		EQU	10000000b

PIC_IRR_READ		EQU	0Ah
PIC_ISR_READ		EQU	0Bh

PIC_EOI			EQU	20h

PIC_SMM_CLEAR		EQU	48h
PIC_SMM_SET		EQU	68h

PIC_EOI_IRQ0		EQU	60h
PIC_EOI_IRQ1		EQU	61h
PIC_EOI_IRQ2		EQU	62h
PIC_EOI_IRQ3		EQU	63h
PIC_EOI_IRQ4		EQU	64h
PIC_EOI_IRQ5		EQU	65h
PIC_EOI_IRQ6		EQU	66h
PIC_EOI_IRQ7		EQU	67h

PIC_EOI_IRQ8		EQU	60h
PIC_EOI_IRQ9		EQU	61h
PIC_EOI_IRQ10		EQU	62h
PIC_EOI_IRQ11		EQU	63h
PIC_EOI_IRQ12		EQU	64h
PIC_EOI_IRQ13		EQU	65h
PIC_EOI_IRQ14		EQU	66h
PIC_EOI_IRQ15		EQU	67h

IRQ0			EQU	0
IRQ1			EQU	1
IRQ2			EQU	2
IRQ3			EQU	3
IRQ4			EQU	4
IRQ5			EQU	5
IRQ6			EQU	6
IRQ7			EQU	7
IRQ8			EQU	8
IRQ9			EQU	9
IRQ10			EQU	10
IRQ11			EQU	11
IRQ12			EQU	12
IRQ13			EQU	13
IRQ14			EQU	14
IRQ15			EQU	15

;---------------------------------------------------------------------------
; 8237 DMA Controller symbols for readability
;---------------------------------------------------------------------------

DMA_MODE_VERIFY		EQU	00000000b
DMA_MODE_READ		EQU	00000100b
DMA_MODE_WRITE		EQU	00001000b
DMA_MODE_AUTOINIT	EQU	00010000b
DMA_MODE_DECREMENT	EQU	00100000b
DMA_MODE_DEMAND		EQU	00000000b
DMA_MODE_SINGLE		EQU	01000000b
DMA_MODE_BLOCK		EQU	10000000b
DMA_MODE_CASCADE	EQU	11000000b

DMA0			EQU	0
DMA1			EQU	1
DMA2			EQU	2
DMA3			EQU	3
DMA4			EQU	4
DMA5			EQU	5
DMA6			EQU	6
DMA7			EQU	7

;---------------------------------------------------------------------------
; Unsupport Instruction(s) -- TASM v5.0
;---------------------------------------------------------------------------

ICEBP	MACRO
	DB	0F1h
	ENDM

INVALID	MACRO
	DB	00Fh, 0B9h
	ENDM

UD1	MACRO
	DB	00Fh, 0B9h
	ENDM

UD2	MACRO
	DB	00Fh, 00Bh
	ENDM

RDTSC	MACRO
	DB	00Fh, 031h
	ENDM

SIDTD	MACRO	???arg:REQ
	DB	66h
	sidt	???arg
	ENDM

SGDTD	MACRO	???arg:REQ
	DB	66h
	sgdt	???arg
	ENDM

LIDTD	MACRO	???arg:REQ
	DB	66h
	lidt	???arg
	ENDM

LGDTD	MACRO	???arg:REQ
	DB	66h
	lgdt	???arg
	ENDM

;---------------------------------------------------------------------------
; V86 Stuff
;---------------------------------------------------------------------------

V86ret	MACRO
	int	41h
	ENDM

;---------------------------------------------------------------------------
; Understandable way of doing *Prefetch Queue* flush
;---------------------------------------------------------------------------

FLUSH_PREFETCH_QUEUE MACRO
	jmp	$+2
	ENDM

;---------------------------------------------------------------------------
; Optimization
;---------------------------------------------------------------------------

PUSHR	MACRO	regop:REQ, memop:REQ
	mov	regop, memop
	push	regop
	ENDM

;---------------------------------------------------------------------------
; For primitive debugging
;---------------------------------------------------------------------------

DBWR	MACRO	char, attr

IFNB	<char>
	mov	BYTE PTR ds:[0B80A0h], char
ELSE
	mov	BYTE PTR ds:[0B80A0h], "T"
ENDIF

IFNB	<attr>
	mov	BYTE PTR ds:[0B80A1h], attr
ELSE
	mov	BYTE PTR ds:[0B80A1h], 1Fh
ENDIF
	ENDM

;---------------------------------------------------------------------------
; Write to unused ISA port for back-to-back I/O delay
;---------------------------------------------------------------------------

IO_DELAY MACRO
	out	0EBh, al
	ENDM
