;---------------------------------------------------------------------------
;
; 			8237 DMA Controller Support
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- ISA_DMA_MaskChannel
;	- ISA_DMA_UnMaskChannel
;	- ISA_DMA_StartTransfer
;	- ISA_DMA_ClearTransfer
;
;---------------------------------------------------------------------------

	INCLUDE std.inc
	INCLUDE stdgroup.inc

	.386p

	PUBLIC	dmaMaskChannel
	PUBLIC	dmaUnMaskChannel
	PUBLIC	dmaStartTransfer
	PUBLIC	dmaClearTransfer

codeSegment SEGMENT

;---------------------------------------------------------------------------
;	dmaMaskChannel
;
;	Entry:
;	      	al = DMA channel #
;	Exit:
;		None
;
DefProc dmaMaskChannel

	push	eax
	push	edx

	movzx	eax, al
	movzx	edx, dmaMask[eax]

	or	al, 00000100b
	out	dx, al

	pop	edx
	pop	eax
	ret

EndProc dmaMaskChannel

;---------------------------------------------------------------------------
;	dmaUnMaskChannel
;
;	Entry:
;	      	al = DMA channel #
;	Exit:
;		None
;
DefProc dmaUnMaskChannel

	push	eax
	push	edx

	movzx	eax, al
	movzx	edx, dmaMask[eax]

	and	al, 00000011b
	out	dx, al

	pop	edx
	pop	eax
	ret

EndProc dmaUnMaskChannel

;---------------------------------------------------------------------------
;	dmaStartTransfer
;
;	Entry:
;	      	al = DMA channel #
;		ah = transfer mode
;	       ebx = transfer address
;	       ecx = transfer length
;	Exit:
;		None
;
DefProc dmaStartTransfer

	push	eax
	push	edx
	push	esi

	movzx	esi, al			; esi = DMA channel #

	cmp	al, DMA3
	jbe	dmaStartTransferMain

	shr	ecx, 1			; adjust count for WORD transfer
	shr	bx, 1			; adjust physical address as well

dmaStartTransferMain:

	dec	ecx			; count is null-based

	movzx	edx, dmaMask[esi]	;
	or	al, 00000100b		; mask channel
	out	dx, al			;

	mov	dl, dmaClear[esi]	;
	mov	al, NULL		; clear channel
	out	dx, al			;

	mov	dl, dmaMode[esi]	;
	or	eax, esi		; set mode
	or	al, ah			;
	out	dx, al			;

	mov	dl, dmaPage[esi]	;
	mov	eax, ebx		; set page
	shr	eax, 16			;
	out	dx, al			;

	mov	dl, dmaAddr[esi]	;
	mov	al, bl			; set page offset LSB
	out	dx, al			;
	mov	al, bh			; set page offset MSB
	out	dx, al			;

	mov	dl, dmaCount[esi]	;
	mov	al, cl			; set transfer length LSB
	out	dx, al			;
	mov	al, ch			; set transfer length MSB
	out	dx, al			;

	mov	dl, dmaMask[esi]	;
	mov	eax, esi		; unmask channel
	out	dx, al			;

	pop	esi
	pop	edx
	pop	eax
	ret

EndProc dmaStartTransfer

;---------------------------------------------------------------------------
;	dmaClearTransfer
;
;	Entry:
;	      	al = DMA channel #
;	Exit:
;		None
;
DefProc dmaClearTransfer

	push	eax
	push	edx
	push	esi

	movzx	esi, al			;
	movzx	edx, dmaMask[esi]	;
	or	al, 00000100b		; mask channel
	out	dx, al			;

	mov	dl, dmaClear[esi]	;
	mov	al, NULL		; clear channel
	out	dx, al			;

	mov	eax, esi		;
	mov	dl, dmaMask[esi]	;
	and	al, 00000011b		; unmask channel
	out	dx, al			;

	pop	esi
	pop	edx
	pop	eax
	ret

EndProc dmaClearTransfer

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

;---------------------------------------------------------------------------
; Lookup table to simplify talking to the DMA controller
;---------------------------------------------------------------------------

dmaMask		DB	00Ah, 00Ah, 00Ah, 00Ah, 0D4h, 0D4h, 0D4h, 0D4h
dmaMode		DB	00Bh, 00Bh, 00Bh, 00Bh, 0D6h, 0D6h, 0D6h, 0D6h
dmaClear	DB	00Ch, 00Ch, 00Ch, 00Ch, 0D8h, 0D8h, 0D8h, 0D8h
dmaPage		DB	087h, 083h, 081h, 082h, 08Fh, 08Bh, 089h, 08Ah
dmaAddr		DB	000h, 002h, 004h, 006h, 0C0h, 0C4h, 0C8h, 0CCh
dmaCount	DB	001h, 003h, 005h, 007h, 0C2h, 0C6h, 0CAh, 0CEh

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

bssSegment ENDS

	END

