;---------------------------------------------------------------------------
;
; 		      Real-Time Clock Support Routines
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- ISA_RTC_Read
;	- ISA_RTC_Write
;	- ISA_RTC_GetTime
;	- ISA_RTC_SetTime
;	- ISA_RTC_GetDate
;	- ISA_RTC_SetDate
;	- ISA_RTC_GetAlarm
;	- ISA_RTC_SetAlarm
;	- ISA_RTC_GetTimerFreq
;	- ISA_RTC_SetTimerFreq
;	- ISA_RTC_GetTimerHandler
;	- ISA_RTC_SetTimerHandler
;
;---------------------------------------------------------------------------

	INCLUDE std.inc
	INCLUDE stdgroup.inc
	INCLUDE	rtc.inc

	.386p

	PUBLIC	rtcRead
	PUBLIC	rtcWrite
	PUBLIC	rtcGetTime
	PUBLIC	rtcSetTime
	PUBLIC	rtcGetDate
	PUBLIC	rtcSetDate
	PUBLIC	rtcGetAlarm
	PUBLIC	rtcSetAlarm

codeSegment SEGMENT

;---------------------------------------------------------------------------
;	rtcWaitUIP - Wait for the Update-In-Progress flag to clear
;
;	Entry:
;	      	None
;	Exit:
;		None
;
DefProc rtcWaitUIP

	push	eax
	push	ecx

	mov	ecx, 00100000h

	mov	al, RTC_STATUS_A
	out	RTC_INDEX_PORT, al

rtcWaitUIPMore:
	in	al, RTC_DATA_PORT
	test	al, RTC_UIP
	loopnz	rtcWaitUIPMore

	pop	ecx
	pop	eax
	ret	

EndProc rtcWaitUIP

;---------------------------------------------------------------------------
;	rtcRead - 
;
;	Entry:
;		ah = RTC register
;	Exit:
;		al = data
;
DefProc rtcRead

	pushfd

	cli

	call	rtcWaitUIP

	mov	al, ah
	out	RTC_INDEX_PORT, al
	in	al, RTC_DATA_PORT

	popfd
	ret

EndProc rtcRead

;---------------------------------------------------------------------------
;	rtcWrite - 
;
;	Entry:
;		ah = RTC register
;		al = data
;	Exit:
;		None
;
DefProc rtcWrite

	pushfd
	push	eax

	cli

	call	rtcWaitUIP

	xchg	al, ah
	out	RTC_INDEX_PORT, al

	xchg	al, ah
	out	RTC_DATA_PORT, al

	pop	eax
	popfd
	ret

EndProc rtcWrite

;---------------------------------------------------------------------------
;	rtcBCDToDec
;
;	Entry:
;		al = value in BCD
;	Exit:
;		al = value in decimal
;
DefProc rtcBCDToDec

	push	edx
	mov	dl, al
        and     al, 00001111b
        shr     dl, 2
	add	al, dl
        shr     dl, 2
	add	al, dl
        shl     al, 1
	pop	edx
	ret

EndProc rtcBCDToDec

;---------------------------------------------------------------------------
;	rtcDecToBCD
;
;	Entry:
;		al = value in decimal
;	Exit:
;		al = value in BCD
;
DefProc rtcDecToBCD

	push	edx
	mov	edx, eax
	aam
	shl	ah, 4
	or	al, ah
	mov	ah, dh
	pop	edx
	ret

EndProc rtcDecToBCD

;---------------------------------------------------------------------------
;	rtcGetTime - 
;
;	Entry:
;		edi = pointer to 3-byte buffer for TIME data:
;
;		  BYTE	Hour
;		  BYTE	Second
;		  BYTE	Minute
;	Exit:
;		None
;
DefProc rtcGetTime

	push	eax
	push	edi

	mov	ah, RTC_SECOND
	call	rtcRead
	call	rtcBCDToDec
	stosb

	mov	ah, RTC_MINUTE
	call	rtcRead
	call	rtcBCDToDec
	stosb

	mov	ah, RTC_HOUR
	call	rtcRead
	call	rtcBCDToDec
	stosb

	clc

	pop	edi
	pop	eax
	ret

EndProc rtcGetTime

;---------------------------------------------------------------------------
;	rtcSetTime - 
;
;	Entry:
;		esi = pointer to 3-byte buffer with TIME data:
;
;		  BYTE	Hour
;		  BYTE	Second
;		  BYTE	Minute
;	Exit:
;		None
;
DefProc rtcSetTime

	push	eax
	push	esi

	lodsb
	mov	ah, RTC_SECOND
	call	rtcDecToBCD
	call	rtcWrite

	lodsb
	mov	ah, RTC_MINUTE
	call	rtcDecToBCD
	call	rtcWrite

	lodsb
	mov	ah, RTC_HOUR
	call	rtcDecToBCD
	call	rtcWrite

	clc

	pop	esi
	pop	eax
	ret

EndProc rtcSetTime

;---------------------------------------------------------------------------
;	rtcGetDate - 
;
;	Entry:
;		edi = pointer to 4-byte buffer for DATE data:
;
;		  BYTE	Day
;		  BYTE	Month
;		  WORD	Year
;	Exit:
;		None
;
DefProc rtcGetDate

	push	eax
	push	edi

	mov	ah, RTC_DAY
	call	rtcRead
	call	rtcBCDToDec
	stosb

	mov	ah, RTC_MONTH
	call	rtcRead
	call	rtcBCDToDec
	stosb

	mov	ah, RTC_YEAR
	call	rtcRead
	call	rtcBCDToDec
	stosb

	mov	ah, RTC_CENTURY
	call	rtcBCDToDec
	call	rtcRead
	stosb

	clc

	pop	edi
	pop	eax
	ret

EndProc rtcGetDate

;---------------------------------------------------------------------------
;	rtcSetDate - 
;
;	Entry:
;		esi = pointer to 4-byte buffer with DATE data:
;
;		  BYTE	Day
;		  BYTE	Month
;		  WORD	Year
;	Exit:
;		None
;
DefProc rtcSetDate

	push	eax
	push	esi

	lodsb
	mov	ah, RTC_DAY
	call	rtcDecToBCD
	call	rtcWrite

	lodsb
	mov	ah, RTC_MONTH
	call	rtcDecToBCD
	call	rtcWrite

	lodsb
	mov	ah, RTC_YEAR
	call	rtcDecToBCD
	call	rtcWrite

	lodsb
	mov	ah, RTC_CENTURY
	call	rtcDecToBCD
	call	rtcWrite

	clc

	pop	esi
	pop	eax
	ret

EndProc rtcSetDate

;---------------------------------------------------------------------------
;	rtcGetAlarm - 
;
;	Entry:
;		edi = pointer to 3-byte buffer for ALARM data:
;
;		  BYTE	Hour
;		  BYTE	Second
;		  BYTE	Minute
;	Exit:
;		None
;
DefProc rtcGetAlarm

	push	eax
	push	edi

	mov	ah, RTC_ALARM_SECOND
	call	rtcRead
	call	rtcBCDToDec
	stosb

	mov	ah, RTC_ALARM_MINUTE
	call	rtcRead
	call	rtcBCDToDec
	stosb

	mov	ah, RTC_ALARM_HOUR
	call	rtcRead
	call	rtcBCDToDec
	stosb

	clc

	pop	edi
	pop	eax
	ret

EndProc rtcGetAlarm

;---------------------------------------------------------------------------
;	rtcSetAlarm - 
;
;	Entry:
;		esi = pointer to 3-byte buffer with ALARM data:
;
;		  BYTE	Hour
;		  BYTE	Second
;		  BYTE	Minute
;	Exit:
;		None
;
DefProc rtcSetAlarm

	push	eax
	push	esi

	lodsb
	mov	ah, RTC_ALARM_SECOND
	call	rtcDecToBCD
	call	rtcWrite

	lodsb
	mov	ah, RTC_ALARM_MINUTE
	call	rtcDecToBCD
	call	rtcWrite

	lodsb
	mov	ah, RTC_ALARM_HOUR
	call	rtcDecToBCD
	call	rtcWrite

	clc

	pop	esi
	pop	eax
	ret

EndProc rtcSetAlarm

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

bssSegment ENDS

	END
