;---------------------------------------------------------------------------
;
; 			32-bit PM Exception Handlers
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- KERNEL_ExpGetHandlerPM
;	- KERNEL_ExpSetHandlerPM
;	- KERNEL_ExpGetHandlerVM
;	- KERNEL_ExpSetHandlerVM
;
;---------------------------------------------------------------------------

	INCLUDE	std.inc
	INCLUDE	stdgroup.inc
	INCLUDE	kernel.api

	.386p

	; Exports

	PUBLIC	expGetHandlerPM
	PUBLIC	expSetHandlerPM
	PUBLIC	expGetHandlerVM
	PUBLIC	expSetHandlerVM

	; Startup/Shutdown

	PUBLIC	expInit
	PUBLIC	expDone

codeSegment SEGMENT

	EXTERN	keFatalExit:NEAR
	EXTERN	idtSetEntry:NEAR

;---------------------------------------------------------------------------
; Exception 00 - Divide Error
;---------------------------------------------------------------------------

exp00Handler:
	push	NULL
	push	00h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 01 - Debug Exception
;---------------------------------------------------------------------------

exp01Handler:

	; ignore debug exceptions -- disable all breakpoints

	push	eax

	xor	eax, eax
	mov	dr6, eax
	mov	dr7, eax

	pop	eax

	iretd

;---------------------------------------------------------------------------
; Exception 02 - NMI
;---------------------------------------------------------------------------

exp02Handler:
	push	NULL
	push	02h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 03 - Break Point
;---------------------------------------------------------------------------

exp03Handler:

	; ignore debug break points -- INT3s

	iretd

;---------------------------------------------------------------------------
; Exception 04 - Overflow
;---------------------------------------------------------------------------

exp04Handler:
	push	NULL
	push	04h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 05 - Bound
;---------------------------------------------------------------------------

exp05Handler:
	push	NULL
	push	05h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 06 - Invalid Opcode
;---------------------------------------------------------------------------

exp06Handler:
	push	NULL
	push	06h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 07 - No Math Coprocessor
;---------------------------------------------------------------------------

exp07Handler:
	push	NULL
	push	07h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 08 - Double Fault
;---------------------------------------------------------------------------

exp08Handler:
	push	08h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 09 - Coprocessor Error
;---------------------------------------------------------------------------

exp09Handler:
	push	NULL
	push	09h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 0A - Invalid TSS
;---------------------------------------------------------------------------

exp0AHandler:
	push	0Ah			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 0B - Segment Not Present
;---------------------------------------------------------------------------

exp0BHandler:
	push	0Bh			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 0C - Stack Fault
;---------------------------------------------------------------------------

exp0CHandler:
	push	0Ch			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 0D - General Protection Fault
;---------------------------------------------------------------------------

exp0DHandler:
	push	0Dh			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 0E - Page Fault
;---------------------------------------------------------------------------

exp0EHandler:
	push	0Eh			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 0F - Intel Reserved
;---------------------------------------------------------------------------

exp0FHandler:
	push	NULL
	push	0Fh			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 10 - Floating Point Error
;---------------------------------------------------------------------------

exp10Handler:
	push	NULL
	push	10h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 11 - Alignment Check
;---------------------------------------------------------------------------

exp11Handler:
	push	11h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 12 - Machine Check
;---------------------------------------------------------------------------

exp12Handler:
	push	NULL
	push	12h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 13 - Reserved
;---------------------------------------------------------------------------

exp13Handler:
	push	NULL
	push	13h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 14 - Reserved
;---------------------------------------------------------------------------

exp14Handler:
	push	NULL
	push	14h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 15 - Reserved
;---------------------------------------------------------------------------

exp15Handler:
	push	NULL
	push	15h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 16 - Reserved
;---------------------------------------------------------------------------

exp16Handler:
	push	NULL
	push	16h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 17 - Reserved
;---------------------------------------------------------------------------

exp17Handler:
	push	NULL
	push	17h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 18 - Reserved
;---------------------------------------------------------------------------

exp18Handler:
	push	NULL
	push	18h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 19 - Reserved
;---------------------------------------------------------------------------

exp19Handler:
	push	NULL
	push	19h			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 1A - Reserved
;---------------------------------------------------------------------------

exp1AHandler:
	push	NULL
	push	1Ah			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 1B - Reserved
;---------------------------------------------------------------------------

exp1BHandler:
	push	NULL
	push	1Bh			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 1C - Reserved
;---------------------------------------------------------------------------

exp1CHandler:
	push	NULL
	push	1Ch			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 1D - Reserved
;---------------------------------------------------------------------------

exp1DHandler:
	push	NULL
	push	1Dh			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 1E - Reserved
;---------------------------------------------------------------------------

exp1EHandler:
	push	NULL
	push	1Eh			
	jmp	expGenericHandler

;---------------------------------------------------------------------------
; Exception 1F - Reserved
;---------------------------------------------------------------------------

exp1FHandler:
	push	NULL
	push	1Fh			
;	jmp	expGenericHandler

;---------------------------------------------------------------------------
;	expGenericHandler
;
;	Entry (Stack):
;
;		DWORD	EXP#		(All)
;		DWORD	Error Code	(All)
;		DWORD	EIP		(All)
;		DWORD	CS		(All)
;		DWORD	EFLAGS		(All)
;		DWORD	ESP		(V86/Ring3)
;		DWORD	SS		(V86/Ring3)
;		DWORD	ES		(V86)
;		DWORD	DS		(V86)
;		DWORD	FS		(V86)
;		DWORD	GS		(V86)
;	Exit:
;		None
;
DefProc expGenericHandler

	push	eax
	push	ebp

	lea	ebp, [esp+16]
	mov	eax, [ebp-8]

	testd	[ebp].v86_EFLAGS, EFLAGS_VM
	jz	expGenericMain

	call	expHandlerVMDef[eax*4]	; call VM Handler
	jnc	expGenericExit		;
				
expGenericMain:
	call	expHandlerPMDef[eax*4]	; call PM Handler
	jnc	expGenericExit		;

	jmp	keFatalExit		; Panic!!!
	
expGenericExit:
	pop	ebp
	pop	eax

	add	esp, SIZE EXP_FRAME

	iretd

EndProc expGenericHandler

;---------------------------------------------------------------------------
;	expDefaultHandler - Default Exception Hook Handler
;
;	Entry (Registers):
;
;		eax = EXP#
;		ebp = pointer to V86_FRAME stack frame
;
;	Entry (Stack Frame):
;
;		DWORD	EIP		(All)
;		DWORD	CS		(All)
;		DWORD	EFLAGS		(All)
;		DWORD	SS		(V86/Ring3)
;		DWORD	ESP		(V86/Ring3)
;		DWORD	ES		(V86)
;		DWORD	DS		(V86)
;		DWORD	FS		(V86)
;		DWORD	GS		(V86)
;	Exit:
;		Set CF==1 to indicate exception was not handled
;
DefProc expDefaultHandler
	stc
	ret
EndProc expDefaultHandler

;---------------------------------------------------------------------------
;	expGetHandlerPM
;
;	Entry:
;		al = EXP#
;	Exit:
;	       ebx = pointer to Kernel Mode handler
;
DefProc expGetHandlerPM

	movzx	ebx, al
	mov	ebx, expHandlerPMDef[ebx*4]
	ret

EndProc expGetHandlerPM

;---------------------------------------------------------------------------
;	expSetHandlerPM
;
;	Entry:
;		al = EXP#
;	       ebx = pointer to new Kernel Mode handler
;	Exit:
;	       ebx = pointer to old Kernel Mode handler
;
DefProc expSetHandlerPM

	push	eax
	movzx	eax, al
	xchg	ebx, expHandlerPMDef[eax*4]
	pop	eax
	ret

EndProc expSetHandlerPM

;---------------------------------------------------------------------------
;	expGetHandlerVM
;
;	Entry:
;		al = EXP#
;	Exit:
;	       ebx = pointer to Virtual Mode handler
;
DefProc expGetHandlerVM

	movzx	ebx, al
	mov	ebx, expHandlerVMDef[ebx*4]
	ret

EndProc expGetHandlerVM

;---------------------------------------------------------------------------
;	expSetHandlerVM
;
;	Entry:
;		al = EXP#
;	       ebx = pointer to new Virtual Mode handler
;	Exit:
;	       ebx = pointer to old Virtual Mode handler
;
DefProc expSetHandlerVM

	push	eax
	movzx	eax, al
	xchg	ebx, expHandlerVMDef[eax*4]
	pop	eax
	ret

EndProc expSetHandlerVM

;---------------------------------------------------------------------------
;	expInit
;
;
;	Entry (Registers):
;
;		ebp = pointer to arguments on the stack
;
;	Entry (Stack Arguments):
;
;		DWORD	Base of Library
;		DWORD	Size of Library
;		DWORD	Base of Reclaim Area
;		DWORD	Size of Reclaim Area
;		DWORD	Base of MDT
;		DWORD	Size of MDT (number of entries)
;		DWORD	Base of PSP
;		DWORD	Base of Environment
;
;	Exit:
;		None
;
DefProc expInit

	pushad

	; fill in handler tables with "expDefaultHandler"

	mov	eax, OFFSET expDefaultHandler
	mov	ecx, 32 * 2
	mov	edi, OFFSET expHandlerPMDef
	rep	stosd

	; install all exception handlers

	xor	eax, eax
	mov	ecx, cs
	mov	edx, DESC_INTR_GATE

expInitNext:
	mov	ebx, expProcTblDef[eax*4]
	call	idtSetEntry

	inc	eax
	cmp	eax, 20h
	jb	expInitNext

	popad
	ret

EndProc expInit

;---------------------------------------------------------------------------
;	expDone
;
;	Entry:
;		None
;	Exit:
;		None
;
DefProc expDone
	ret
EndProc expDone

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

; Exception Procs

expProcTblDef	LABEL	DWORD
	DD	exp00Handler, exp01Handler, exp02Handler, exp03Handler
	DD	exp04Handler, exp05Handler, exp06Handler, exp07Handler
	DD	exp08Handler, exp09Handler, exp0AHandler, exp0BHandler
	DD	exp0CHandler, exp0DHandler, exp0EHandler, exp0FHandler
	DD	exp10Handler, exp11Handler, exp12Handler, exp13Handler
	DD	exp14Handler, exp15Handler, exp16Handler, exp17Handler
	DD	exp18Handler, exp19Handler, exp1AHandler, exp1BHandler
	DD	exp1CHandler, exp1DHandler, exp1EHandler, exp1FHandler
expProcTblEnd	LABEL	DWORD

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

; Kernel and V86 Mode exception handlers

expHandlerPMDef	LABEL	DWORD
	DD	32 DUP (?)
expHandlerPMEnd	LABEL	DWORD

expHandlerVMDef	LABEL	DWORD
	DD	32 DUP (?)
expHandlerVMEnd	LABEL	DWORD

bssSegment ENDS

	END
