;---------------------------------------------------------------------------
;
; 		    IRQ handlers & 8259 related routines
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- KERNEL_IrqGetHandlerPM
;	- KERNEL_IrqSetHandlerPM
;	- KERNEL_IrqGetHandlerVM
;	- KERNEL_IrqSetHandlerVM
;	- KERNEL_IrqGetPMIntByIRQ
;	- KERNEL_IrqGetVMIntByIRQ
;
;	The IRQ handers registerred with SetHandlerPM/VM, will be
;	called with the following stack frame (V86_FRAME):
;
;		DWORD	EIP		(All)
;		DWORD	CS		(All)
;		DWORD	EFLAGS		(All)
;		DWORD	SS		(V86/Ring3)
;		DWORD	ESP		(V86/Ring3)
;		DWORD	ES		(V86)
;		DWORD	DS		(V86)
;		DWORD	FS		(V86)
;		DWORD	GS		(V86)
;
;	The registers must be preserved and contains the following:
;
;		eax = EXP#
;		ebp = pointer to V86_FRAME stack frame (above)
;
;
;	8259A PIC Services:
;
;	- KERNEL_IrqGetBase
;	- KERNEL_IrqSetBase
;	- KERNEL_IrqMask
;	- KERNEL_IrqUnMask
;	- KERNEL_IrqEOI
;
;	These PIC related services are typically used when bypassing the
;	built-in IRQ dispatcher mechanism. You might want to do this if
;	you need every clock cycle you can get.
;
;	Another typical use of these services is to call PIC_UnMask just
;	after you have installed/hooked an IRQ handler. The IRQ must be 
;	unmasked for you to receive any interrupts.
;
;	These routines really belong inside ISA.SYS, but for now they
;	will remain where they are.
;
;
;	Notification:
;
;	- KERNEL_IrqComplete
;
;---------------------------------------------------------------------------

	INCLUDE	std.inc
	INCLUDE	stdgroup.inc
	INCLUDE	kernel.api

	.386p

	; Exports

	PUBLIC	irqGetHandlerPM
	PUBLIC	irqSetHandlerPM
	PUBLIC	irqGetHandlerVM
	PUBLIC	irqSetHandlerVM
	PUBLIC	irqGetVMIntByIRQ
	PUBLIC	irqGetPMIntByIRQ

	; 8259A Routines

	PUBLIC	picGetBase
	PUBLIC	picSetBase
	PUBLIC	picMask
	PUBLIC	picUnMask
	PUBLIC	picEOI

	; Notification

	PUBLIC	irqComplete

	; Startup/Shutdown

	PUBLIC	irqInit
	PUBLIC	irqDone

codeSegment SEGMENT

	EXTERN	idtSetEntry:NEAR

;---------------------------------------------------------------------------
; IRQ 00 - System Timer
;---------------------------------------------------------------------------

irq00Handler:
	push	00h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ1 - Keyboard
;---------------------------------------------------------------------------

irq01Handler:
	push	01h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ2 - Cascade
;---------------------------------------------------------------------------

irq02Handler:
	push	02h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ3 - COM2
;---------------------------------------------------------------------------

irq03Handler:
	push	03h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ4 - COM1
;---------------------------------------------------------------------------

irq04Handler:
	push	04h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ5 - LPT2
;---------------------------------------------------------------------------

irq05Handler:
	push	05h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ6 - Floppy
;---------------------------------------------------------------------------

irq06Handler:
	push	06h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ7 - LPT1
;---------------------------------------------------------------------------

irq07Handler:
	push	07h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ8 - Real Time Clock
;---------------------------------------------------------------------------

irq08Handler:
	push	08h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ9 - IRQ 2 Redirect / Misc (PCI/PnP)
;---------------------------------------------------------------------------

irq09Handler:
	push	09h			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ10 - Misc (PCI/PnP)
;---------------------------------------------------------------------------

irq0AHandler:
	push	0Ah			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ11 - Misc (PCI/PnP)
;---------------------------------------------------------------------------

irq0BHandler:
	push	0Bh			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ12 - PS/2 Mouse / Misc (PCI/PnP)
;---------------------------------------------------------------------------

irq0CHandler:
	push	0Ch			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ13 - Coprocessor
;---------------------------------------------------------------------------

irq0DHandler:
	push	0Dh			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ14 - 1st IDE Controller / Misc (PCI/PnP)
;---------------------------------------------------------------------------

irq0EHandler:
	push	0Eh			
	jmp	irqGenericHandler

;---------------------------------------------------------------------------
; IRQ15 - 2nd IDE Controller / Misc (PCI/PnP)
;---------------------------------------------------------------------------

irq0FHandler:
	push	0Fh			
;	jmp	irqGenericHandler

;---------------------------------------------------------------------------
;	irqGenericHandler
;
;	Entry (Stack):
;
;		DWORD	IRQ#		(All)
;		DWORD	EIP		(All)
;		DWORD	CS		(All)
;		DWORD	EFLAGS		(All)
;		DWORD	ESP		(V86/Ring3)
;		DWORD	SS		(V86/Ring3)
;		DWORD	ES		(V86)
;		DWORD	DS		(V86)
;		DWORD	FS		(V86)
;		DWORD	GS		(V86)
;	Exit:
;		IRQ# is discarded, and IRETD is executed.
;
DefProc irqGenericHandler

	push	eax
	push	ebp

	lea	ebp, [esp+12]
	mov	eax, [ebp-4]

	mov	ebp, esp		; ebp -> V86_FRAME

	testd	[ebp].v86_EFLAGS, EFLAGS_VM
	jz	irqGenericMain

	call	irqHandlerVMDef[eax*4]	; call VM Handler
	jnc	irqGenericExit		;
				
irqGenericMain:
	call	irqHandlerPMDef[eax*4]	; call PM Handler
	jnc	irqGenericExit		;

	APIcall	KERNEL_IrqEOI		; signal EOI

irqGenericExit:
	pop	ebp
	pop	eax

	add	esp, SIZE IRQ_FRAME	; discard IRQ#

	iretd

EndProc irqGenericHandler

;---------------------------------------------------------------------------
;	irqDefaultHandler
;
;	Entry (Registers):
;
;		eax = IRQ#
;		ebp = pointer to V86_FRAME stack frame
;
;	Entry (Stack Frame):
;
;		DWORD	EIP		(All)
;		DWORD	CS		(All)
;		DWORD	EFLAGS		(All)
;		DWORD	ESP		(V86/Ring3)
;		DWORD	SS		(V86/Ring3)
;		DWORD	ES		(V86)
;		DWORD	DS		(V86)
;		DWORD	FS		(V86)
;		DWORD	GS		(V86)
;	Exit:
;		Set CF==1 to indicate IRQ was not handled
;
DefProc irqDefaultHandler
	stc
	ret
EndProc irqDefaultHandler

;---------------------------------------------------------------------------
;	irqGetHandlerPM
;
;	Entry:
;		al = IRQ#
;	Exit:
;	       ebx = pointer to 'Kernel Mode' handler
;
DefProc irqGetHandlerPM

	movzx	ebx, al
	mov	ebx, irqHandlerPMDef[ebx*4]
	ret

EndProc irqGetHandlerPM

;---------------------------------------------------------------------------
;	irqSetHandlerPM
;
;	Entry:
;		al = IRQ#
;	       ebx = pointer to new 'Kernel Mode' handler
;	Exit:
;	       ebx = pointer to old 'Kernel Mode' handler
;
DefProc irqSetHandlerPM

	push	eax
	movzx	eax, al
	xchg	ebx, irqHandlerPMDef[eax*4]
	pop	eax
	ret

EndProc irqSetHandlerPM

;---------------------------------------------------------------------------
;	irqGetHandlerVM
;
;	Entry:
;		al = IRQ#
;	Exit:
;	       ebx = pointer to 'Virtual Mode' handler
;
DefProc irqGetHandlerVM

	movzx	ebx, al
	mov	ebx, irqHandlerVMDef[ebx*4]
	ret

EndProc irqGetHandlerVM

;---------------------------------------------------------------------------
;	irqSetHandlerVM
;
;	Entry:
;		al = IRQ#
;	       ebx = pointer to new 'Virtual Mode' handler
;	Exit:
;	       ebx = pointer to old 'Virtual Mode' handler
;
DefProc irqSetHandlerVM

	push	eax
	movzx	eax, al
	xchg	ebx, irqHandlerVMDef[eax*4]
	pop	eax
	ret

EndProc irqSetHandlerVM

;---------------------------------------------------------------------------
;	irqGetPMIntByIRQ
;
;	Entry:
;		al = IRQ #
;	Exit:
;		al = PM Interrupt #
;
DefProc irqGetPMIntByIRQ
	
	cmp	al, IRQ15
	ja	irqGetPMIntByIRQFail

	push	edx

	mov	dl, irqBasePM[0]
	cmp	al, IRQ7
	jna	irqGetPMIntByIRQDone

	mov	dl, irqBasePM[1]
	sub	al, IRQ8

irqGetPMIntByIRQDone:
	add	al, dl

	pop	edx
	clc
	ret

irqGetPMIntByIRQFail:
	mov	al, 0FFh
	stc
	ret

EndProc irqGetPMIntByIRQ

;---------------------------------------------------------------------------
;	irqGetVMIntByIRQ
;
;	Entry:
;		al = IRQ #
;	Exit:
;		al = VM Interrupt #
;
DefProc irqGetVMIntByIRQ
	
	cmp	al, IRQ15
	ja	irqGetVMIntByIRQFail

	push	edx

	mov	dl, irqBaseVM[0]
	cmp	al, IRQ7
	jna	irqGetVMIntByIRQDone

	mov	dl, irqBaseVM[1]
	sub	al, IRQ8

irqGetVMIntByIRQDone:
	add	al, dl
	clc
	pop	edx
	ret

irqGetVMIntByIRQFail:
	stc
	ret

EndProc irqGetVMIntByIRQ

;---------------------------------------------------------------------------
;	picGetBase
;
;	Entry:
;		None
;	Exit:
;		bl = 1st 8259 IRQ base
;		bh = 2nd 8259 IRQ base
;
DefProc picGetBase

	mov	bx, WORD PTR irqBasePM
	ret

EndProc picGetBase

;---------------------------------------------------------------------------
;	picSetBase
;
;	Entry:
;		bl = 1st 8259 IRQ base
;		bh = 2nd 8259 IRQ base
;	Exit:
;		None
;
DefProc picSetBase

	pushfd	
	push	eax

	cli

	mov	WORD PTR irqBasePM, bx	; save for future use

	mov	al, PIC_ICW1		; ICW1
	out	PIC_PORT20, al		;
	out	PIC_PORTA0, al		;

	mov	al, bl			; ICW2 - master IRQ base
	out	PIC_PORT21, al		;
	mov	al, bh			; ICW2 - slave IRQ base
	out	PIC_PORTA1, al		;

	mov	al, PIC_ICW3_A		; ICW3 - slave connected on IRQ2
	out	PIC_PORT21, al		;
	mov	al, PIC_ICW3_B		; ICW3 - slave connected on IRQ2
	out	PIC_PORTA1, al		;

	mov	al, PIC_ICW4		; ICW4 - unbuffered, 80x86 mode
	out	PIC_PORT21, al		;
	out	PIC_PORTA1, al		;

	pop	eax
	popfd
	ret

EndProc picSetBase

;---------------------------------------------------------------------------
;	picMask - physically mask the specified 8259 IRQ
;
;	Entry:
;		al = IRQ #
;	Exit:
;		None
;
DefProc picMask

	cmp	al, IRQ15		; invalid IRQ?
	ja	picMaskFail		;

	pushfd
	push	eax
	push	ecx

	mov	cl, al			; cl = IRQ 
	and	cl, 00000111b		; cl = IRQ & 7 (0-7)
	mov	ah, 00000001b		; ah = bit mask for IRQ0
	shl	ah, cl			; ah = correct mask bit

	cli

	cmp	al, IRQ8		; slave IRQ?
	jb	picMaskSkip		;

	in	al, PIC_PORTA1		; read slave IMR
	or	al, ah			; mask specified IRQ
	out	PIC_PORTA1, al		; write slave IMR

	cmp	al, PIC_IMR_ALL		; all slave IRQs masked?
	jne	picMaskDone		;

	mov	ah, PIC_IMR_IRQ2	; mask off master IRQ2

picMaskSkip:
	in	al, PIC_PORT21		; read master IMR
	or	al, ah			; mask specified IRQ
	out	PIC_PORT21, al		; write master IMR

picMaskDone:
	clc
	pop	ecx
	pop	eax
	popfd
	ret

picMaskFail:
	stc
	ret

EndProc picMask

;---------------------------------------------------------------------------
;	picUnMask - physically unmask the specified 8259 IRQ
;
;	Entry:
;		al = IRQ #
;	Exit:
;		None
;
DefProc picUnMask

	cmp	al, IRQ15		; invalid IRQ?
	ja	picUnMaskFail		;

	pushfd
	push	eax
	push	ecx

	mov	cl, al			; cl = IRQ 
	and	cl, 00000111b		; cl = IRQ & 7 (0-7)
	mov	ah, 00000001b		; ah = bit mask for IRQ0
	shl	ah, cl			; ah = correct mask bit
	not	ah			; ah = inverted for unmasking

	cli

	cmp	al, IRQ8		; slave IRQ?
	jb	picUnMaskSkip		;

	in	al, PIC_PORTA1		; read slave IMR
	and	al, ah			; unmask specified IRQ
	out	PIC_PORTA1, al		; write slave IMR

	mov	ah, NOT PIC_IMR_IRQ2	; unmask master IRQ2

picUnMaskSkip:
	in	al, PIC_PORT21		; read master IMR
	and	al, ah			; unmask specified IRQ
	out	PIC_PORT21, al		; write master IMR

	clc

	pop	ecx
	pop	eax
	popfd
	ret

picUnMaskFail:
	stc
	ret

EndProc picUnMask

;---------------------------------------------------------------------------
;	picEOI - send non-specific EOI
;
;	Entry:
;		al = IRQ #
;	Exit:
;		None
;
DefProc picEOI

	cmp	al, IRQ15		; invalid IRQ?
	ja	picEOIFail		;

	pushfd
	push	eax

	cli

	cmp	al, IRQ8		; slave IRQ?
	jb	picEOISkip		;

	mov	al, PIC_EOI		; EOI command
	out	PIC_PORTA0, al		; slave EOI

picEOISkip:
	mov	al, PIC_EOI		; EOI command
	out	PIC_PORT20, al		; master EOI

	clc

	pop	eax
	popfd
	ret

picEOIFail:
	stc
	ret

EndProc picEOI

;---------------------------------------------------------------------------
;	irqComplete
;
;	Entry:
;		None
;	Exit:
;		None
;
DefProc irqComplete
	ret
EndProc irqComplete

;---------------------------------------------------------------------------
;	irqInit
;
;
;	Entry (Registers):
;
;		ebp = pointer to arguments on the stack
;
;	Entry (Stack Arguments):
;
;		DWORD	Base of Library
;		DWORD	Size of Library
;		DWORD	Base of Reclaim Area
;		DWORD	Size of Reclaim Area
;		DWORD	Base of MDT
;		DWORD	Size of MDT (number of entries)
;		DWORD	Base of PSP
;		DWORD	Base of Environment
;
;	Exit:
;		None
;
DefProc irqInit

	pushad

	in	al, PIC_PORT21
	mov	irqMaskReg[0], al
	in	al, PIC_PORTA1
	mov	irqMaskReg[1], al

	; set kernel PIC mappings

	mov	bl, STD_PIC_BASE_1
	mov	bh, STD_PIC_BASE_2
	call	picSetBase

	; fill in handler tables with "irqDefaultHandler"

	mov	eax, OFFSET irqDefaultHandler
	mov	ecx, 16 * 2
	mov	edi, OFFSET irqHandlerPMDef
	rep	stosd

	mov	eax, STD_PIC_BASE_1
	mov	ecx, cs
	mov	edx, DESC_INTR_GATE
	xor	esi, esi

irqInitNext:
	mov	ebx, irqProcTblDef[esi*4]
	call	idtSetEntry

	inc	esi
	inc	eax
	cmp	eax, STD_PIC_BASE_1 + 16
	jb	irqInitNext

	popad
	ret

EndProc irqInit

;---------------------------------------------------------------------------
;	irqDone
;
;	Entry:
;		None
;	Exit:
;		None
;
DefProc irqDone

	push	eax
	push	ebx

	; set IBM compatible PIC mappings (Interrupts 08h/70h)

	mov	bl, IBM_PIC_BASE_1
	mov	bh, IBM_PIC_BASE_2
	call	picSetBase

	mov	al, irqMaskReg[0]
	out	PIC_PORT21, al
	mov	al, irqMaskReg[1]
	out	PIC_PORTA1, al

	pop	ebx
	pop	eax
	ret

EndProc irqDone

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

; IRQ Procs

irqProcTblDef	LABEL	DWORD
	DD	irq00Handler, irq01Handler, irq02Handler, irq03Handler
	DD	irq04Handler, irq05Handler, irq06Handler, irq07Handler
	DD	irq08Handler, irq09Handler, irq0AHandler, irq0BHandler
	DD	irq0CHandler, irq0DHandler, irq0EHandler, irq0FHandler
irqProcTblEnd	LABEL	DWORD

irqBasePM	DB	STD_PIC_BASE_1, STD_PIC_BASE_2
irqBaseVM	DB	IBM_PIC_BASE_1, IBM_PIC_BASE_2

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

; Kernel and V86 Mode IRQ handlers

irqHandlerPMDef	LABEL	DWORD
	DD	16 DUP (?)
irqHandlerPMEnd	LABEL	DWORD

irqHandlerVMDef	LABEL	DWORD
	DD	16 DUP (?)
irqHandlerVMEnd	LABEL	DWORD

irqMaskReg	DB	2 DUP (?)

bssSegment ENDS

	END
