;---------------------------------------------------------------------------
;
; 		 Portable Executable "PE" Support Routines
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- KERNEL_LoadModule
;	- KERNEL_FreeModule
;
;---------------------------------------------------------------------------

	INCLUDE	std.inc
	INCLUDE	stdgroup.inc
	INCLUDE	mzexe.inc
	INCLUDE	peexe.inc
	INCLUDE	kernel.api

	.386p

	; Exports

	PUBLIC	peLoadModule
	PUBLIC	peFreeModule

codeSegment SEGMENT

;---------------------------------------------------------------------------
;	peLoadModule
;
;	Entry:
;		esi = pointer to ASCIIZ module name
;	Exit:
;		eax = image entry point
;		ebx = image base
;		ecx = image size
;
DefProc peLoadModule

	push	edx
	push	esi
	push	edi

	;-------------------------------------------------------------
	; open "PE"
	;-------------------------------------------------------------

	APIcall	KERNEL_IfsFileOpen
	jc	peLoadModuleFail

	mov	ebx, eax
	mov	edi, OFFSET ourTempBuf

	;-------------------------------------------------------------
	; read "MZ" header
	;-------------------------------------------------------------

	xor	eax, eax
	APIcall	KERNEL_IfsFileSetPos

	mov	ecx, SIZE MZ_EXE
	mov	edx, edi
	APIcall	KERNEL_IfsFileRead

	cmp	[edi].mzSignature, MZ_EXE_MAGIC
	jne	peLoadModuleFail

	;-------------------------------------------------------------
	; read "PE" header
	;-------------------------------------------------------------

	mov	eax, [edi].mzNewExeHeader
	APIcall	KERNEL_IfsFileSetPos

	mov	ecx, SIZE PE_EXE
	mov	edx, edi
	APIcall	KERNEL_IfsFileRead

	cmp	[edi].peSignature, PE_EXE_MAGIC
	jne	peLoadModuleFail

	;-------------------------------------------------------------
	; allocate kernel memory and clear memory block
	;-------------------------------------------------------------

	mov	ecx, [edi].peImageSize
	APIcall	KERNEL_PageAllocate
	jc	peLoadModuleFail

	push	eax
	push	edi

	mov	edi, eax
	xor	eax, eax
	shr	ecx, 2
	rep	stosd

	pop	edi
	pop	edx

	;-------------------------------------------------------------
	; read entire image header
	;-------------------------------------------------------------

	xor	eax, eax
	APIcall	KERNEL_IfsFileSetPos

	mov	ecx, [edi].peHeaderSize
	APIcall	KERNEL_IfsFileRead

	mov	edi, edx
	add	edi, [edx].mzNewExeHeader

	movzx	ecx, [edi].peObjectCount
	movzx	esi, [edi].peNTHeaderSize

	add	esi, peOptionalStart
	add	esi, edi

	;-------------------------------------------------------------
	; read all sections into memory
	;-------------------------------------------------------------

peLoadModuleCopy:
	push	ecx
	push	edx

	mov	eax, [esi].peObjPhysicalOfs
	APIcall	KERNEL_IfsFileSetPos

	add	edx, [esi].peObjRVA
	mov	ecx, [esi].peObjPhysicalSize
	APIcall	KERNEL_IfsFileRead

	pop	edx
	pop	ecx

	add	esi, SIZE PE_OBJ
	dec	ecx
	jnz	peLoadModuleCopy

	;-------------------------------------------------------------
	; close file, relocate and execute image
	;-------------------------------------------------------------

	APIcall	KERNEL_IfsFileClose

	mov	ebx, edx

	call	peRelocate
	call	peImport

	mov	ecx, [edi].peImageSize
	mov	eax, [edi].peEntryPoint
	add	eax, ebx

peLoadModuleExit:
	pop	edi
	pop	esi
	pop	edx
	ret

peLoadModuleFail:
	stc
	jmp	peLoadModuleExit	

EndProc peLoadModule

;---------------------------------------------------------------------------
;	peFreeModule
;
;	Entry:
;		ebx = pointer to "PE" image - 'module handle'
;	Exit:
;		None
;
DefProc peFreeModule
	ret
EndProc peFreeModule

;---------------------------------------------------------------------------
;	peRelocate
;
;	Entry:
;		ebx = pointer to "PE" image - 'module handle'
;	Exit:
;		None
;
DefProc peRelocate

	pushad

	mov	edi, ebx
	add	edi, [edi].mzNewExeHeader

	mov	edx, ebx

	sub	edx, [edi].peImageBase
	jz	peRelocateDone

	mov	[edi].peImageBase, ebx

	mov	esi, [edi].peFixupRVA
	mov	ecx, [edi].peFixupSize
	jecxz	peRelocateDone

peRelocateMore:
	mov	ebp, [ebx][esi].peFixPageRVA
	mov	ecx, [ebx][esi].peFixBlockSize
	jecxz	peRelocateDone

	sub	ecx, 8
	add	esi, 8

peRelocateNext:
	mov	eax, [ebx][esi]
	mov	edi, eax

	and	ah, 11110000b
	cmp	ah, 00110000b
	jne	peRelocateSkip

	and	edi, 00000FFFh
	add	edi, ebp
	add	[ebx][edi], edx

peRelocateSkip:
	inc	esi
	inc	esi
	dec	ecx
	dec	ecx
	jnz	peRelocateNext
	jmp	peRelocateMore

peRelocateDone:
	clc
	popad
	ret

EndProc peRelocate

;---------------------------------------------------------------------------
;	peImport
;
;	Entry:
;		ebx = pointer to "PE" image - 'module handle'
;	Exit:
;		None
;
DefProc peImport
	ret
EndProc peImport

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

ourTempBuf	DB	SIZE PE_EXE DUP (?)

bssSegment ENDS

	END
