;---------------------------------------------------------------------------
;
;		      'Run-Time Library'-style Routines
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- KERNEL_ListGetHead
;	- KERNEL_ListGetTail
;	- KERNEL_ListGetPrev
;	- KERNEL_ListGetNext
;	- KERNEL_ListAttachHead
;	- KERNEL_ListAttachTail
;	- KERNEL_ListDetachHead
;	- KERNEL_ListDetachTail
;	- KERNEL_ListInsert
;	- KERNEL_ListDelete
;
;	- KERNEL_StringCopy	
;	- KERNEL_StringAppend
;	- KERNEL_StringInsert
;	- KERNEL_StringDelete
;	- KERNEL_StringUpper	
;	- KERNEL_StringLower	
;	- KERNEL_StringNumber
;	- KERNEL_StringFormat
;
;---------------------------------------------------------------------------

	INCLUDE	std.inc
	INCLUDE	stdgroup.inc
	INCLUDE	kernel.api

	.386p

	; Publics

	PUBLIC	rtlListGetHead
	PUBLIC	rtlListGetTail
	PUBLIC	rtlListGetPrev
	PUBLIC	rtlListGetNext
	PUBLIC	rtlListAttachHead
	PUBLIC	rtlListAttachTail
	PUBLIC	rtlListDetachHead
	PUBLIC	rtlListDetachTail
	PUBLIC	rtlListInsert
	PUBLIC	rtlListDelete

	PUBLIC	rtlStringCopy
	PUBLIC	rtlStringAppend
	PUBLIC	rtlStringInsert
	PUBLIC	rtlStringDelete
	PUBLIC	rtlStringUpper
	PUBLIC	rtlStringLower
	PUBLIC	rtlStringNumber
;	PUBLIC	rtlStringFormat

codeSegment SEGMENT

	; Externs

;===========================================================================
;
;			Doubly Linked List Management
;
;===========================================================================

;---------------------------------------------------------------------------
;	rtlListGetHead
;
;	Entry:
;		ebx = List Handle
;	Exit:
;		eax = Item
;
DefProc rtlListGetHead

 	mov	eax, [ebx].listHead
	cmp	eax, 1
	ret

EndProc rtlListGetHead

;---------------------------------------------------------------------------
;	rtlListGetTail
;
;	Entry:
;		ebx = List Handle
;	Exit:
;		eax = Item
;
DefProc rtlListGetTail

 	mov	eax, [ebx].listTail
	cmp	eax, 1
	ret

EndProc rtlListGetTail

;---------------------------------------------------------------------------
;	rtlListGetPrev
;
;	Entry:
;		eax = Item (Current)
;	Exit:
;		eax = Item (Previous)
;
DefProc rtlListGetPrev

	cmp	eax, 1
	jb	rtlListGetPrevExit

 	mov	eax, [eax].listPrev
	cmp	eax, 1

rtlListGetPrevExit:
	ret

EndProc rtlListGetPrev

;---------------------------------------------------------------------------
;	rtlListGetNext
;
;	Entry:
;		eax = Item (Current)
;	Exit:
;		eax = Item (Next)
;
DefProc rtlListGetNext

	cmp	eax, 1
	jb	rtlListGetNextExit

 	mov	eax, [eax].listNext
	cmp	eax, 1

rtlListGetNextExit:
	ret

EndProc rtlListGetNext

;---------------------------------------------------------------------------
;	rtlListAttachHead
;
;	Entry:
;		eax = Item
;		ebx = List Handle
;	Exit:
;	    	None
;
DefProc rtlListAttachHead

	push	edx

	mov	edx, [ebx].listHead
	test	edx, edx
	jnz	rtlListAttachHeadMain

rtlListAttachHeadInit:
	mov	[eax].listPrev, edx	; Item.Prev = NULL
	mov	[eax].listNext, edx	; Item.Next = NULL
	mov	[ebx].listTail, eax	; List.Tail = Item
	mov	[ebx].listHead, eax	; List.Head = Item

	jmp	rtlListAttachHeadExit

rtlListAttachHeadMain:
	mov	[eax].listNext, edx	; Item.Next = (Old).List.Head
	mov	[ebx].listHead, eax	; List.Head = Item
	mov	[edx].listPrev, eax	; (Old).List.Head.Prev = Item
	mov	[eax].listPrev, NULL	; Item.Prev = NULL

rtlListAttachHeadExit:
	pop	edx
	ret

EndProc rtlListAttachHead

;---------------------------------------------------------------------------
;	rtlListAttachTail
;
;	Entry:
;		eax = Item
;		ebx = List Handle
;	Exit:
;	    	None
;
DefProc rtlListAttachTail

	push	edx

	mov	edx, [ebx].listTail
	test	edx, edx
	jz	rtlListAttachHeadInit	;* interproc-jmp

rtlListAttachTailMain:
	mov	[eax].listPrev, edx	; Item.Prev = (Old).List.Tail
	mov	[ebx].listTail, eax	; List.Tail = Item
	mov	[edx].listNext, eax	; (Old).List.Tail.Next = Item
	mov	[eax].listNext, NULL	; Item.Next = NULL

rtlListAttachTailExit:
	pop	edx
	ret

EndProc rtlListAttachTail

;---------------------------------------------------------------------------
;	rtlListDetachHead
;
;	Entry:
;		ebx = List Handle
;	Exit:
;	    	None
;
DefProc rtlListDetachHead

	push	eax

	call	rtlListGetHead
	jc	rtlListDetachHeadExit

	call	rtlListGetNext
	mov	[ebx].listHead, eax
	jc	rtlListDetachHead
	mov	[eax].listPrev, NULL

rtlListDetachHeadExit:
	pop	eax
	ret

EndProc rtlListDetachHead

;---------------------------------------------------------------------------
;	rtlListDetachTail
;
;	Entry:
;		ebx = List Handle
;	Exit:
;	    	None
;
DefProc rtlListDetachTail

	push	eax

	call	rtlListGetTail
	jc	rtlListDetachTailExit

	call	rtlListGetPrev
	mov	[ebx].listTail, eax
	jc	rtlListDetachTailExit
	mov	[eax].listNext, NULL

rtlListDetachTailExit:
	pop	eax
	ret

EndProc rtlListDetachTail

;---------------------------------------------------------------------------
;	rtlListInsert
;
;	Entry:
;		eax = List Item
;		ebx = List Handle
;		esi = List Item (New)
;	Exit:
;	    	None
;
DefProc rtlListInsert

	push	eax
	push	edx

	mov	edx, eax

	call	rtlListGetHead
	jc	rtlListInsertExit

rtlListInsertFind:
	cmp	eax, edx
	jne	rtlListInsertSkip

	cmp	[eax].listPrev, NULL
	je	rtlListInsertHead

	cmp	[eax].listNext, NULL
	je	rtlListInsertTail

	mov	edx, [eax].listNext
	mov	[eax].listNext, esi
	mov	[esi].listPrev, eax
	mov	[edx].listPrev, esi
	mov	[esi].listNext, edx

	clc
	jmp	rtlListInsertExit

rtlListInsertSkip:
	call	rtlListGetNext
	jnc	rtlListInsertFind

rtlListInsertExit:
	pop	edx
	pop	eax
	ret

rtlListInsertHead:
	mov	eax, esi
	call	rtlListAttachHead
	jmp	rtlListInsertExit
	
rtlListInsertTail:
	mov	eax, esi
	call	rtlListAttachTail
	jmp	rtlListInsertExit

EndProc rtlListInsert

;---------------------------------------------------------------------------
;	rtlListDelete
;
;	Entry:
;		eax = List Item
;		ebx = List Handle
;	Exit:
;	    	None
;
DefProc rtlListDelete

	push	eax
	push	edx

	mov	edx, eax

	call	rtlListGetHead
	jc	rtlListDeleteExit

rtlListDeleteFind:
	cmp	eax, edx
	jne	rtlListDeleteSkip

	cmp	[eax].listPrev, NULL
	je	rtlListDeleteHead

	cmp	[eax].listNext, NULL
	je	rtlListDeleteTail

	mov	edx, [eax].listNext
	mov	eax, [eax].listPrev
	mov	[eax].listNext, edx
	mov	[edx].listPrev, eax

	clc
	jmp	rtlListDeleteExit

rtlListDeleteSkip:
	call	rtlListGetNext
	jnc	rtlListDeleteFind

rtlListDeleteExit:
	pop	edx
	pop	eax
	ret

rtlListDeleteHead:
	call	rtlListDetachHead
	jmp	rtlListDeleteExit
	
rtlListDeleteTail:
	call	rtlListDetachTail
	jmp	rtlListDeleteExit

EndProc rtlListDelete

;===========================================================================
;
;		   Null-Terminated String Management
;
;===========================================================================

;---------------------------------------------------------------------------
;	rtlStringCopy
;
;	Entry:
;		esi = pointer to source string
;		edi = pointer to target buffer
;	Exit:
;		edi = pointer to target buffer null-terminator
;
DefProc rtlStringCopy

	push	eax
	push	esi

	cld

rtlStringCopyNext:
	lodsb				; get soruce character
	stosb				;
	test	al, al			; null-termiantor?
	jnz	rtlStringCopyNext	;

	dec	edi

	pop	esi
	pop	eax
	ret

EndProc rtlStringCopy

;---------------------------------------------------------------------------
;	rtlStringAppend
;
;	Entry:
;		esi = pointer to source string
;		edi = pointer to target string
;	Exit:
;	    	None
;
DefProc rtlStringAppend

	push	eax
	push	esi
	push	edi

	xchg	esi, edi		; esi =

	cld

rtlStringAppendNext:
	lodsb				; get soruce character
	test	al, al			; null-termiantor?
	jnz	rtlStringAppendNext	;

	xchg	esi, edi

	dec	edi

	call	rtlStringCopy

	pop	edi
	pop	esi
	pop	eax
	ret

EndProc rtlStringAppend

;---------------------------------------------------------------------------
;	rtlStringInsert
;
;	Entry:
;		esi = pointer to source string
;		edi = pointer to target string
;		ecx = insertion point in target string
;	Exit:
;	    	None
;
DefProc rtlStringInsert
	ret
EndProc rtlStringInsert

;---------------------------------------------------------------------------
;	rtlStringDelete
;
;	Entry:
;		edi = pointer to target string
;		ebx = delete position
;		ecx = delete count
;	Exit:
;	    	None
;
DefProc rtlStringDelete
	ret
EndProc rtlStringDelete

;---------------------------------------------------------------------------
;	rtlStringUpper
;
;	Entry:
;		edi = pointer to target string
;	Exit:
;	    	None
;
DefProc rtlStringUpper

	push	eax
	push	edi

rtlStringUpperNext:
	mov	al, [edi]
	test	al, al
	jz	rtlStringUpperDone

	cmp	al, "a"
	jb	rtlStringUpperSkip	
	cmp	al, "z"
	ja	rtlStringUpperSkip

	sub	al, " "

rtlStringUpperSkip:
	inc	edi
	jmp	rtlStringUpperNext

rtlStringUpperDone:	
	pop	edi
	pop	eax
	ret

EndProc rtlStringUpper

;---------------------------------------------------------------------------
;	rtlStringLower
;
;	Entry:
;		edi = pointer to target string
;	Exit:
;	    	None
;
DefProc rtlStringLower

	push	eax
	push	edi

rtlStringLowerNext:
	mov	al, [edi]
	test	al, al
	jz	rtlStringLowerDone

	cmp	al, "A"
	jb	rtlStringLowerSkip	
	cmp	al, "A"
	ja	rtlStringLowerSkip

	add	al, " "

rtlStringLowerSkip:
	inc	edi
	jmp	rtlStringLowerNext

rtlStringLowerDone:	
	pop	edi
	pop	eax
	ret

EndProc rtlStringLower

;---------------------------------------------------------------------------
;	rtlStringNumber
;
;	Entry:
;		eax = number (signed/unsigned)
;		 bl = number base
;		 cl = number width
;		edi = buffer where to store number
;	Exit:
;		edi = pointer to target buffer null-terminator
;
DefProc rtlStringNumber

	push	eax
	push	ebx
	push	ecx
	push	edx

	movzx	ebx, bl
	movzx	ecx, cl

	push	LARGE 0

rtlStringNumberNext:
	xor	edx, edx
	div	ebx

	add	dl, "0"
	cmp	dl, "9"
	jna	rtlStringNumberSkip
	add	dl, "A" - "9" - 1

rtlStringNumberSkip:
	push	edx
	test	ecx, ecx
	jz	rtlStringNumberAuto

	dec	ecx
	jnz	rtlStringNumberNext
	jmp	rtlStringNumberCopy

rtlStringNumberAuto:
	test	eax, eax
	jnz	rtlStringNumberNext

rtlStringNumberCopy:
	pop	eax
	stosb
	test	eax, eax
	jnz	rtlStringNumberCopy

	dec	edi

	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	ret

EndProc rtlStringNumber

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

bssSegment ENDS

	END

