;---------------------------------------------------------------------------
;
; 		     	PCI Device Access and Enumeration
;
;	Copyright (c) 1999 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- PCI_GetVersion
;
;	Config Space Access:
;
;	- PCI_ReadByte
;	- PCI_ReadWord
;	- PCI_ReadDWord
;	- PCI_WriteByte
;	- PCI_WriteWord
;	- PCI_WriteDWord
;
;	Searching/Enumeration routines:
;
;	- PCI_FindFirst
;	- PCI_FindNext
;	- PCI_FindFirstByID
;	- PCI_FindNextByID
;	- PCI_FindFirstByClass
;	- PCI_FindNextByClass
;
;---------------------------------------------------------------------------

	INCLUDE std.inc
	INCLUDE stdgroup.inc
	INCLUDE	pci.inc
	INCLUDE	kernel.api
	INCLUDE	pci.api

	.386p

codeSegment SEGMENT

;---------------------------------------------------------------------------
;	pciGetVersion - 
;
;	Entry:
;		None
;	Exit:
;		eax = version ID
;
DefProc pciGetVersion

	mov	eax, 00000100h
	clc
	ret

EndProc pciGetVersion

;---------------------------------------------------------------------------
;	pciSetPFA - Set PCI Function Address
;
;	Entry:
;	       ecx = PCI Config Address:
;
;		 ecx[24:16] = Bus (0..255)
;		 ecx[15:11] = Device (0..31)
;		 ecx[10:08] = Function (0..7)
;		 ecx[07:00] = Address (CL = BYTE offset)
;	Exit:
;		edx = Data Register
;
DefProc pciSetPFA

	push	eax
	mov	edx, PCI_CFG_ADDR
	mov	eax, 80000000h
	or	eax, ecx
	and	eax, 80FFFFFCh
	out	dx, eax
	mov	dl, 0FCh
	or	dl, cl
	pop	eax
	ret

EndProc pciSetPFA

;---------------------------------------------------------------------------
;	pciReadByte
;
;	Entry:
;	       ecx = PCI Config Address:
;
;		 ecx[24:16] = Bus (0..255)
;		 ecx[15:11] = Device (0..31)
;		 ecx[10:08] = Function (0..7)
;		 ecx[07:00] = Address (CL = BYTE offset)
;	Exit:
;		al = BYTE read
;
DefProc pciReadByte

	push	edx
	call	pciSetPFA
	in	al, dx
	pop	edx	
	ret

EndProc pciReadByte

;---------------------------------------------------------------------------
;	pciReadWord
;
;	Entry:
;	       ecx = PCI Config Address:
;
;		 ecx[24:16] = Bus (0..255)
;		 ecx[15:11] = Device (0..31)
;		 ecx[10:08] = Function (0..7)
;		 ecx[07:01] = Address (CL = WORD aligned offset)
;	Exit:
;		ax = WORD read
;
DefProc pciReadWord

	push	edx
	call	pciSetPFA
	in	ax, dx
	pop	edx	
	ret

EndProc pciReadWord

;---------------------------------------------------------------------------
;	pciReadDWord
;
;	Entry:
;	       ecx = PCI Config Address:
;
;		 ecx[24:16] = Bus (0..255)
;		 ecx[15:11] = Device (0..31)
;		 ecx[10:08] = Function (0..7)
;		 ecx[07:02] = Address (CL = DWORD aligned offset)
;	Exit:
;		eax = DWORD read
;
DefProc pciReadDWord

	push	edx
	call	pciSetPFA
	in	eax, dx
	pop	edx	
	ret

EndProc pciReadDWord

;---------------------------------------------------------------------------
;	pciWriteByte
;
;	Entry:
;		al = BYTE to write
;	       ecx = PCI Config Address:
;
;		 ecx[24:16] = Bus (0..255)
;		 ecx[15:11] = Device (0..31)
;		 ecx[10:08] = Function (0..7)
;		 ecx[07:00] = Address (CL = BYTE offset)
;	Exit:
;		None
;
DefProc pciWriteByte

	push	edx
	call	pciSetPFA
	out	dx, al
	pop	edx	
	ret

EndProc pciWriteByte

;---------------------------------------------------------------------------
;	pciWriteWord
;
;	Entry:
;		ax = WORD to write
;	       ecx = PCI Config Address:
;
;		 ecx[24:16] = Bus (0..255)
;		 ecx[15:11] = Device (0..31)
;		 ecx[10:08] = Function (0..7)
;		 ecx[07:01] = Address (CL = WORD aligned offset)
;	Exit:
;		None
;
DefProc pciWriteWord

	push	edx
	call	pciSetPFA
	out	dx, ax
	pop	edx	
	ret

EndProc pciWriteWord

;---------------------------------------------------------------------------
;	pciWriteDWord
;
;	Entry:
;	       eax = DWORD to write
;	       ecx = PCI Config Address:
;
;		 ecx[24:16] = Bus (0..255)
;		 ecx[15:11] = Device (0..31)
;		 ecx[10:08] = Function (0..7)
;		 ecx[07:01] = Address (CL = DWORD aligned offset)
;	Exit:
;		None
;
DefProc pciWriteDWord

	push	edx
	call	pciSetPFA
	out	dx, eax
	pop	edx	
	ret

EndProc pciWriteDWord

;---------------------------------------------------------------------------
;	pciFindFirst
;
;	Entry:
;		None
;	Exit:
;		eax = VID/DID
;
;		   eax[31:16] = Device ID
;		   eax[15:00] = Vendor ID
;
;		ecx = PFA of *first* PCI device
;
DefProc pciFindFirst

	xor	ecx, ecx
	jmp	pciFindNextMain

EndProc pciFindFirst

;---------------------------------------------------------------------------
;	pciFindNext
;
;	Entry:
;		ecx = PFA of *intermediate* PCI device
;	Exit:
;		eax = VID/DID
;
;		   eax[31:16] = Device ID
;		   eax[15:00] = Vendor ID
;
;		ecx = PFA of *next* PCI device
;
;		CF==1 if there are no more PCI devices
;
DefProc pciFindNext

	mov	cl, PCI_REG_HEADER_TYPE
	call	pciReadByte

	test	al, PCI_MULTI_FUNCTION
	jnz	pciFindNextFunc

pciFindNextSkip:
	and	ecx, 00FFF800h
	add	ecx, 00000800h
	jmp	pciFindNextTest

pciFindNextFunc:
	add	ecx, 00000100h

pciFindNextTest:
	and	ecx, 00FFFF00h
	jnz	pciFindNextMain

	stc
	jmp	pciFindNextExit

pciFindNextMain:
	call	pciReadDWord

	test	eax, eax
	jz	pciFindNextSkip

	cmp	eax, -1
	je	pciFindNextSkip

	clc

pciFindNextExit:
	ret

EndProc pciFindNext

;---------------------------------------------------------------------------
;	pciFindFirstByID
;
;	Entry:
;		eax = VID/DID
;
;		   eax[31:16] = Device ID
;		   eax[15:00] = Vendor ID
;	Exit:
;		ecx = PFA of first matching PCI device
;
DefProc pciFindFirstByID

	push	eax
	push	edx

	mov	edx, eax

	call	pciFindFirst
	jc	pciFindNextByIDExit

	jmp	pciFindNextByIDMain

EndProc pciFindFirstByID

;---------------------------------------------------------------------------
;	pciFindNextByID
;
;	Entry:
;		eax = VID/DID
;
;		   eax[31:16] = Device ID
;		   eax[15:00] = Vendor ID
;
;		ecx = PFA of previously matching PCI device
;	Exit:
;		ecx = PFA of next matching PCI device
;
DefProc pciFindNextByID

	push	eax
	push	edx

	mov	edx, eax

pciFindNextByIDNext:
	call	pciFindNext
	jc	pciFindNextByIDExit

pciFindNextByIDMain:
	cmp	eax, edx
	jne	pciFindNextByIDNext

pciFindNextByIDExit:
	pop	edx
	pop	eax
	ret

EndProc pciFindNextByID

;---------------------------------------------------------------------------
;	pciFindFirstByClass
;
;	Entry:
;		 al = class
;	Exit:
;		ecx = PFA of first matching PCI device
;
DefProc pciFindFirstByClass

	push	eax
	push	edx

	mov	edx, eax

	call	pciFindFirst
	jc	pciFindNextByClassExit

	jmp	pciFindNextByClassMain

EndProc pciFindFirstByClass

;---------------------------------------------------------------------------
;	pciFindNextByClass
;
;	Entry:
;		 ah = class
;		 al = sub-class
;		ecx = PFA of previously matching PCI device
;	Exit:
;		ecx = PFA of next matching PCI device
;
DefProc pciFindNextByClass

	push	eax
	push	edx

	mov	edx, eax

pciFindNextByClassNext:
	call	pciFindNext
	jc	pciFindNextByClassExit

pciFindNextByClassMain:
	mov	cl, PCI_REG_SUB_CLASS
	call	pciReadWord

	cmp	ax, dx
	jne	pciFindNextByClassNext

pciFindNextByClassExit:
	pop	edx
	pop	eax
	ret

EndProc pciFindNextByClass

;---------------------------------------------------------------------------
;	mainEntryPoint - PCI.SYS entry point
;
;	Entry (Stack):
;
;		DWORD = return address
;
;	Entry (Registers):
;
;		ebx = our memory block base -- "module handle"
;		ecx = our memory block size
;
DefProc mainEntryPoint

	mov	esi, OFFSET apiServiceTbl
	APIcall	KERNEL_ApiRegister

	clc
	ret

EndProc mainEntryPoint

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

;---------------------------------------------------------------------------
; PCI.SYS API Service Table
;---------------------------------------------------------------------------

apiServiceTbl	LABEL	DWORD
	API_TABLE	<apiServiceDef, (apiServiceEnd-apiServiceDef)/8>
apiServiceDef	LABEL	DWORD
	API_ENTRY	<PCI_GetVersion,	pciGetVersion>
	API_ENTRY	<PCI_ReadByte,		pciReadByte>
	API_ENTRY	<PCI_ReadWord,		pciReadWord>
	API_ENTRY	<PCI_ReadDWord,		pciReadDWord>
	API_ENTRY	<PCI_WriteByte,		pciWriteByte>
	API_ENTRY	<PCI_WriteWord,		pciWriteWord>
	API_ENTRY	<PCI_WriteDWord,	pciWriteDWord>
	API_ENTRY	<PCI_FindFirst,		pciFindFirst>
	API_ENTRY	<PCI_FindNext,		pciFindNext>
	API_ENTRY	<PCI_FindFirstByID,	pciFindFirstByID>
	API_ENTRY	<PCI_FindNextByID,	pciFindNextByID>
	API_ENTRY	<PCI_FindFirstByClass,	pciFindFirstByClass>
	API_ENTRY	<PCI_FindNextByClass,	pciFindNextByClass>
apiServiceEnd	LABEL	DWORD

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

bssSegment ENDS

	END	mainEntryPoint
