;---------------------------------------------------------------------------
;
;		       Ring0/Ring3/V86 Thread Support
;
;     Copyright (c) 1999-2000 by Thomas Kjoernes, all rights reserved.
;
;---------------------------------------------------------------------------

	INCLUDE std.inc
	INCLUDE stdgroup.inc

	.386p

	; Publics

	PUBLIC	threadTimerTick

codeSegment SEGMENT

	; Externs

;---------------------------------------------------------------------------
;	threadTimerTick - 
;
;	Entry:
;		ss:esp = Ring0 Stack (Interrupt Stack Frame)
;	Exit:
;		None
;
threadTimerTick PROC NEAR

	push	edi
	push	ds

	push	ss
	pop	ds

	; ToDo: edi -> Current Thread

	jmp	threadTimerTickExit

	;-------------------------------------------------------------
	; save common CPU state
	;-------------------------------------------------------------

	mov	[edi].threadEAX, eax
	mov	[edi].threadEBX, ebx
	mov	[edi].threadECX, ecx
	mov	[edi].threadEDX, edx
	mov	[edi].threadEBP, ebp
	mov	[edi].threadESI, esi

	mov	[edi].threadES, es
	mov	[edi].threadFS, fs
	mov	[edi].threadGS, gs

	pop	[edi].threadDS
	pop	[edi].threadEDI

	pop	ebp			; pop "threadTimerTick" RET addr

	pop	[edi].threadEIP
	pop	[edi].threadCS
	pop	[edi].threadEFLAGS

	; ToDo: esi -> New Thread

	;-------------------------------------------------------------
	; save V86 specific CPU state
	;-------------------------------------------------------------

	testd	[edi].threadEFLAGS, EFLAGS_VM
	jz	threadTimerTickGetRing3

	pop	[edi].threadESP
	pop	[edi].threadSS
	pop	[edi].threadES
	pop	[edi].threadDS
	pop	[edi].threadFS
	pop	[edi].threadGS

	jmp	threadTimerTickGetCommon

	;-------------------------------------------------------------
	; save Ring3 specific CPU state
	;-------------------------------------------------------------

threadTimerTickGetRing3:

	testd	[edi].threadCS, RPL3
	jz	threadTimerTickGetRing0

	pop	[edi].threadESP
	pop	[edi].threadSS

	jmp	threadTimerTickGetCommon

	;-------------------------------------------------------------
	; save Ring0 specific CPU state
	;-------------------------------------------------------------

threadTimerTickGetRing0:

	mov	[edi].threadESP, esp
	mov	[edi].threadSS, ss

threadTimerTickGetCommon:

	;-------------------------------------------------------------
	; restore V86 specific CPU state
	;-------------------------------------------------------------

	testd	[esi].threadEFLAGS, EFLAGS_VM
	jz	threadTimerTickSetRing3

	xor	eax, eax

	push	[esi].threadGS
	push	[esi].threadFS
	push	[esi].threadDS
	push	[esi].threadES
	push	[esi].threadSS
	push	[esi].threadESP

	mov	[esi].threadDS, eax
	mov	[esi].threadES, eax
	mov	[esi].threadFS, eax
	mov	[esi].threadGS, eax

	jmp	threadTimerTickSetCommon

	;-------------------------------------------------------------
	; restore Ring3 specific CPU state
	;-------------------------------------------------------------

threadTimerTickSetRing3:	

	testd	[esi].threadCS, RPL3
	jz	threadTimerTickSetRing0

	push	[esi].threadSS
	push	[esi].threadESP

	jmp	threadTimerTickSetCommon

	;-------------------------------------------------------------
	; restore Ring0 specific CPU state
	;-------------------------------------------------------------

threadTimerTickSetRing0:

	lss	esp, FWORD PTR [esi].threadESP

threadTimerTickSetCommon:

	;-------------------------------------------------------------
	; restore common CPU state
	;-------------------------------------------------------------

	push	[esi].threadEFLAGS
	push	[esi].threadCS
	push	[esi].threadEIP

	push	ebp			; push "threadTimerTick" RET addr

	push	[esi].threadEDI
	push	[esi].threadDS

 	mov	es, [esi].threadES
 	mov	fs, [esi].threadFS
	mov	gs, [esi].threadGS

	mov	eax, [esi].threadEAX
	mov	ebx, [esi].threadEBX
	mov	ecx, [esi].threadECX
	mov	edx, [esi].threadEDX
	mov	ebp, [esi].threadEBP
	mov	esi, [esi].threadESI

threadTimerTickExit:
	pop	ds
	pop	edi
	ret

threadTimerTick ENDP

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

bssSegment ENDS

	END

