{=---------------------------------------------------------------------=}
{                                                                       }
{            PESHRINK -- PE Executable File Shrinker v1.0a              }
{                                                                       }
{   Copyright (c) 1999-2000 by Thomas Kjoernes, all rights reserved.    }
{                                                                       }
{=---------------------------------------------------------------------=}

{$S-}
{$R-}
{$I-}
{$G+}

PROGRAM PE_EXE_Shrinker;

TYPE
  TMZHeader = RECORD
    Signature: Word;
    PagesMod: Word;
    PagesDiv: Word;
    RelocCount: Word;
    HeaderSize: Word;
    MinMem: Word;
    MaxMem: Word;
    InitSS: Word;
    InitSP: Word;
    Checksum: Word;
    EntryIP: Word;
    EntryCS: Word;
    RelocTable: Word;
    OverlayNum: Word;
    Reserved: ARRAY [1..28] OF Char;
    LibDirPtr: Longint;
    NewExePtr: Longint;
  END;

  TPEHeader = RECORD
    peSignature: Longint;
    peCPUType: Word;
    peObjectCount: Word;
    peTimeStamp: ARRAY [1..3] OF Longint;
    peNTHeaderSize: Word;
    peFlags: Word;
    peOptionalStart: ARRAY [1..4] OF Longint;
    peEntryPoint: Longint;
    peReserved: ARRAY [1..2] OF Longint;
    peImageBase: Longint;
    peObjectAlign: Longint;
    peFileAlign: Longint;
    peVersionInfo: ARRAY [1..8] OF Word;
    peImageSize: Longint;
    peHeaderSize: Longint;
    peCheckSum: Longint;
    peSubSys: Word;
    peDLLFlags: Word;
    peStackReserve: Longint;
    peStackCommit: Longint;
    peHeapReserve: Longint;
    peHeapCommit: Longint;
    peFill: Longint;
    peRVAs: Longint;
    peExportRVA: Longint;
    peExportSize: Longint;
    peImportRVA: Longint;
    peImportSize: Longint;
    peResourceRVA: Longint;
    peResourceSize: Longint;
    peExceptionRVA: Longint;
    peExceptionSize: Longint;
    peSecurityRVA: Longint;
    peSecuritySize: Longint;
    peFixupRVA: Longint;
    peFixupSize: Longint;
    peDebugRVA: Longint;
    peDebugSize: Longint;
    peImageDescRVA: Longint;
    peImageDescSize: Longint;
    peMachineRVA: Longint;
    peMachineSize: Longint;
    peTLSRVA: Longint;
    peTLSSize: Longint;
  END;

  TPEObject = RECORD
    peObjName: ARRAY [1..8] OF Char;
    peObjVirtualSize: Longint;
    peObjRVA: Longint;
    peObjPhysicalSize: Longint;
    peObjPhysicalOfs: Longint;
    peObjReserved: ARRAY [1..3] OF Longint;
    peObjFlags: Longint;
  END;

VAR
  MZHeader: TMZHeader;
  PEHeader: TPEHeader;
  PEObject: TPEObject;
  SFile: FILE;
  SPath: String;
  L: Longint;
  Z: Longint;
  I: Integer;
  Dummy: Char;

PROCEDURE ShrinkExe;
BEGIN

  SPath := ParamStr(1);

  Assign(SFile, SPath);

  {$I-}
  Reset(SFile, 1);
  {$I+}

  IF IOResult <> 0 THEN
  BEGIN
    WriteLn('- Unable to open EXE "',SPath,'"');
    Halt(1);
  END;

  BlockRead(SFile, MZHeader, SizeOf(MZHeader));

  IF MZHeader.Signature <> $5A4D THEN
  BEGIN
    WriteLn('- File is not a valid "MZ" Executable!');
    Halt(2);
  END;

  IF (MZHeader.RelocTable < $0040) OR (MZHeader.NewExePtr = 0) THEN
  BEGIN
    WriteLn('- File is not a valid "PE" Executable!');
    Halt(2);
  END;

  Seek(SFile, MZHeader.NewExePtr);

  BlockRead(SFile, PEHeader, SizeOf(PEHeader));

  Seek(SFile, MZHeader.NewExePtr + PEHeader.peNTHeaderSize + 24);

  L := 0;

  FOR I := 1 TO PEHeader.peObjectCount DO
  BEGIN
    BlockRead(SFile, PEObject, SizeOf(PEObject));
    IF PEObject.peObjPhysicalOfs > L THEN
    BEGIN
      L := PEObject.peObjPhysicalOfs;
      Z := PEObject.peObjPhysicalSize;
    END;
  END;

  IF L <> 0 THEN
  BEGIN
    Seek(SFile, L+Z);
    Truncate(SFile);
  END;

  Close(SFile);

END;

BEGIN
  IF ParamCount = 0 THEN
  BEGIN
    WriteLn('PESHRINK - PE Executable File Shrinker v1.0a');
    WriteLn('Copyright (c) 1999-2000 by Thomas Kjoernes, all rights reserved.');
    WriteLn;
    WriteLn('  Usage: PESHRINK [filename.EXE]');
  END ELSE ShrinkExe;
END.
